;
; DOS DUMP UTILITY FOR C1P/UK101
;
; DUMPS THE DOS FROM AN OS65D DISK
;
; USES RAW READ TO GET DOS PART 1 FROM TRACK 0
; AND DOS CALLS TO GET DOS PART 2 FROM TRACK 1
; 
; HISTORY:
;
; 0.2  INCREASED DELAYS FOR USE ON FASTER
;      MACHINES. DOS ENTRY POINTS CHANGED
;      FOR BETTER COMPATIBILITY WITH
;      OLDER DOS VERSIONS.
;
; 0.11 FIXED ERROR COUNT. CHANGED BUFFER
;      BUFFER SIZE TO 9 PAGES AND FILL CHAR
;      TO $00.
;
; 0.1  1ST RELEASE
;

ORG	=	$0400

MAXPAG	=	9	; BUFFER SIZE (PAGES)
FILLCH	=	$00	; FILL CHARACTER

BUFFER	=	$4000	; TRACK BUFFER

; OSI MONITOR ROM ROUTINES

LOADF	=	$203	; LOAD FLAG
SAVEF	=	$205	; SAVE FLAG
INPUT	=	$FFEB	; GET CHAR FROM KEYBOARD/ACIA
OUTPUT	=	$FFEE	; PUT CHAR TO SCREEN/ACIA

; OSI DOS ROUTINES

SECNUM	=	$265E	; SECTOR# TO READ/WRITE (1-8)
PAGES	=	$265F	; #PAGES READ/TO WRITE (1-8)
MEM	=	$2660	; MEMORY TO READ/WRITE
TRKNUM	=	$2662	; TRACK# (1-39)
GOTRK	=	$26A6	; GO TO TRACK
LODHED	=	$2754	; LOAD HEAD
UNLODH	=	$2761	; UNLOAD HEAD
READSEC	=	$295D	; READ SECTOR TO MEMORY

DISK	=	$C000	; DISK CONTROLLER

CR	=	$0D
LF	=	$0A

	*=	ORG

	LDA	#0
	STA	LOADF	; LOAD OFF
	STA	SAVEF	; SAVE OFF

	LDX	#<HELLO
	LDA	#>HELLO
	JSR	PRINT

	LDX	#<INSDISK
	LDA	#>INSDISK
	JSR	PRINT
	JSR	ANYKEY

	LDA	#1	; SAVE ON
	STA	SAVEF
	JSR	INITPIA

; GET DOS FROM TRACK 0

	LDA	#0
	JSR	READMSG
	JSR	GETDOS
	JSR	SAVMSG
	LDA	PAGCNT
	JSR	SAVBUF
	JSR	SHOWERR

	LDX	#<DOSADR
	LDA	#>DOSADR
	JSR	PRINT
	LDA	LODADR+1
	LDX	LODADR
	PHA
	JSR	HEX2OUT
	LDX	#<DOSPAG
	LDA	#>DOSPAG
	JSR	PRINT
	LDA	PAGCNT
	JSR	HEXOUT
	JSR	CRLF
	PLA
	CMP	#$22
	BEQ	DOSOK

	LDA	#0	; SAVE OFF
	STA	SAVEF
	LDX	#<BADDOS
	LDA	#>BADDOS
	JSR	PRINT
	JMP	QUIT2

; MOVE DOS TO CORRECT POSITION

DOSOK	JSR	RESPTR
	LDA	LODADR
	STA	$FB
	LDA	LODADR+1
	STA	$FC
	LDX	PAGCNT
	LDY	#0
LOOP1	LDA	($FD),Y
	STA	($FB),Y
	INY
	BNE	LOOP1
	INC	$FE
	INC	$FC
	DEX
	BNE	LOOP1

; GET DOS FROM TRACK 1

	LDA	#1
	JSR	READMSG
	JSR	INIT
	LDA	#1	; TRACK 1
	STA	SECNUM	; SECTOR 1
	STA	TRKNUM	; TRACK 1
	JSR	GOTRK	; GO TO TRACK
	LDA	#<BUFFER
	STA	MEM
	LDA	#>BUFFER
	STA	MEM+1
	JSR	LODHED	; LOAD HEAD
	JSR	READSEC	; READ SECTOR
	JSR	UNLODH	; UNLOAD HEAD
	JSR	SAVMSG
	LDA	PAGES
	JSR	SAVBUF

QUIT	LDA	#0	; SAVE OFF
	STA	SAVEF
	LDX	#<DONE
	LDA	#>DONE
	JSR	PRINT
QUIT2	JSR	INPUT
	JMP	($FFFC)

; READING TRACK (A) MSG

READMSG	PHA
	LDX	#<READING
	LDA	#>READING
	JSR	PRINT
	PLA
	CLC
	ADC	#1
	JMP	HEXOUT

; SAVING MSG

SAVMSG	LDX	#<SAVING
	LDA	#>SAVING
	JMP	PRINT

; SAVE BUFFER CONTENTS (A)

SAVBUF	PHA
	JSR	RESPTR
	PLA
	STA	$FF
	LDA	#0
	STA	CNT
	STA	CNT+1
	TAY
	TAX
	BEQ	SAVBUF4

SAVBUF1	CPX	#0
	BNE	SAVBUF2
	JSR	CRLF
	LDA	CNT+1
	LDX	CNT
	JSR	HEX2OUT
	LDA	#':'
	JSR	OUTPUT
	LDX	#16
SAVBUF2	LDA	#' '
	JSR	OUTPUT
	LDA	($FD),Y
	JSR	HEXOUT
	INC	CNT
	BNE	SAVBUF3
	INC	CNT+1
SAVBUF3	DEX
	INY
	BNE	SAVBUF1
	INC	$FE
	DEC	$FF
SAVBUF4	LDA	$FF
	BNE	SAVBUF1
	JMP	CRLF

;

SHOWERR	LDX	#<ERRORS
	LDA	#>ERRORS
	JSR	PRINT
	LDA	ERRCNT+1
	LDX	ERRCNT
	JSR	HEX2OUT
	JSR	CRLF
	RTS

; INIT POINTERS, CLEAR BUFFER & ERROR COUNT

INIT	JSR	RESPTR	; SET POINTERS

	LDA	#0
	STA	ERRCNT
	STA	ERRCNT+1

	TAY
	LDA	#FILLCH
INIT1	STA	($FD),Y
	INY
	BNE	INIT1
	INC	$FE
	DEC	$FF
	BNE	INIT1

; RESET BUFFER POINTERS

RESPTR	LDA	#MAXPAG
	STA	$FF
	LDA	#<BUFFER
	STA	$FD
	LDA	#>BUFFER
	STA	$FE
	RTS

; INIT DISK CONTROLLER PIA

INITPIA	LDY	#0
	STY	DISK+1	; SELECT DDRA
	STY	DISK	; SET PORTA TO ALL INPUTS
	LDX	#4	; 0000 0100
	STX	DISK+1	; SELECT PORTA
	STY	DISK+3	; SELECT DDRB
	DEY
	STY	DISK+2	; SET PORTB TO ALL OUTPUTS
	STX	DISK+3	; SELECT PORTB
	STY	DISK+2	; SET PORTB OUTPUTS HIGH
	RTS

; STEP TO TRACK 0

TZERO	LDA	#$FB	; 1111 1011
	BNE	TZERO3	; DIR=INWARDS

TZERO1	LDA	#2	; 0000 0010
	BIT	DISK	; TEST 'TRK0' PIN
	BNE	TZERO2	; AT TRACK 0
	RTS

TZERO2	LDA	#$FF	; DIR=OUTWARDS (TO TRK0)

TZERO3	STA	DISK+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DISK+2	; SET 'STEP' PIN LOW
	JSR	DELAY2
	ORA	#8	; 0000 1000
	STA	DISK+2	; SET 'STEP' PIN HIGH
	LDX	#$30	; 60MS
	JSR	DELAY
	BEQ	TZERO1	; ALWAYS

; READ TRACK 0 TO BUFFER

GETDOS	JSR	TZERO
	JSR	INIT
	LDX	#$7F	; 0111 1111
	STX	DISK+2	; LOAD DISK HEAD
	LDX	#$FF	; 320MS
	JSR	DELAY
RDTRK1	LDA	DISK	; WAIT START OF
	BMI	RDTRK1	; INDEX PULSE
RDTRK2	LDA	DISK	; WAIT END OF
	BPL	RDTRK2	; INDEX PULSE
	LDA	#3	; RESET ACIA
	STA	DISK+$10
	LDA	#$58	; DIVIDE BY 1, 8E1, RTS HIGH
	STA	DISK+$10
	JSR	RDBYT	; DOS ADDRESS (HIGH)
	BCS	RDTRK4
	STA	LODADR+1
	JSR	RDBYT	; DOS ADDRESS (LOW)
	BCS	RDTRK4
	STA	LODADR
	JSR	RDBYT	; #PAGES TO READ
	BCS	RDTRK4
	STA	PAGCNT
	CMP	#MAXPAG+1 ; IF NONSENSE...
	BCC	RDTRK0
	LDA	#MAXPAG	; LIMIT TRANSFER SIZE
RDTRK0	STA	$FF
	LDY	#0	; READ UNTIL ALL PAGES DONE
RDTRK3	JSR	RDBYT
	BCS	RDTRK4	; OR END-OF-TRACK
	STA	($FD),Y
	INY
	BNE	RDTRK3
	INC	$FE
	DEC	$FF
	BNE	RDTRK3
RDTRK4	LDA	#$FF
	STA	DISK+2	; UNLOAD DISK HEAD
	RTS

; DELAY 1.25MS PER LOOP

DELAY	LDY	#$F8
DELAY1	DEY
	BNE	DELAY1
	EOR	$FF,X
	DEX
	BNE	DELAY
	RTS

DELAY2	JSR	DELAY21
DELAY21	RTS

; INPUT CHAR FROM DISK, CY SET IF END-OF-TRACK

RDBYT	LDA	DISK	; TEST FOR INDEX PULSE
	BPL	RDBYT2
	LDA	DISK+$10
	LSR	A
	BCC	RDBYT
	BIT	DISK+$10 ; TEST PARITY ERROR
	BVS	RDBYT3
	LDA	DISK+$11 ; READ BYTE
	CLC
	RTS

RDBYT2	LDA	#FILLCH
	SEC
	RTS

RDBYT3	INC	ERRCNT
	BNE	RDBYT4
	INC	ERRCNT+1
RDBYT4	LDA	DISK+$11 ; READ BYTE
	CLC
	RTS

; PRINT STRING (A,X)

PRINT	STX	$FD
	STA	$FE
	LDY	#0
PRINT1	LDA	($FD),Y
	BNE	PRINT2
	RTS
PRINT2	JSR	OUTPUT
	INC	$FD
	BNE	PRINT1
	INC	$FE
	JMP	PRINT1

; OUTPUT WORD (A,X) AS HEX

HEX2OUT	JSR	HEXOUT
	TXA

; OUTPUT (A) AS HEX

HEXOUT	PHA
	LSR	A
	LSR	A
	LSR	A
	LSR	A
	JSR	HEXOUT1
	PLA
HEXOUT1	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	HEXOUT2
	CLC
	ADC	#7
HEXOUT2	JMP	OUTPUT

CRLF	LDA	#CR
	JSR	OUTPUT
	LDA	#LF
	JMP	OUTPUT

ANYKEY	LDX	#<ANY
	LDA	#>ANY
	JSR	PRINT
	JMP	INPUT

CNT	.WORD	0
ERRCNT	.WORD	0
PAGCNT	.BYTE	0
LODADR	.WORD	0

ANY	.BYTE	CR,LF
	.BYTE	'PRESS ANY KEY WHEN READY',CR,LF,0

HELLO	.BYTE	CR,LF
	.BYTE	'DOSDUMP 0.2',CR,LF,CR,LF
	.BYTE	'DUMP DOS FROM OS65D DISKS',CR,LF
	.BYTE	'TO CASSETTE',CR,LF,CR,LF
	.BYTE	'WRITE PROTECT ALL DISKS!',CR,LF,0

INSDISK	.BYTE	CR,LF
	.BYTE	'INSERT A DOS DISK THEN',0

READING	.BYTE	CR,LF,CR,LF
	.BYTE	'; READING DOS PART ',0

SAVING	.BYTE	' ...SAVING',CR,LF,0

ERRORS	.BYTE	CR,LF
	.BYTE	'; ERRORS $',0

DONE	.BYTE	CR,LF,CR,LF
	.BYTE	'DONE ... REMOVE DISK',CR,LF,0

DOSADR	.BYTE	'; LOAD $',0

DOSPAG	.BYTE	' PAGES $',0

BADDOS	.BYTE	CR,LF,CR,LF
	.BYTE	'MISSING OR NON-STANDARD',CR,LF
	.BYTE	'DOS ON TRACK 0. CAN''T',CR,LF
	.BYTE	'CONTINUE ... ABORTING',CR,LF,0

	.END
